#include "SimCore.h"
#include <mmsystem.h>

ClothSim::ClothSim()
: Counter( 0 )
{
	Grids.resize( GRIDNUM );
	for( int i = 0; i < GRIDNUM; ++i ) Grids[i].resize( GRIDNUM );

	for( int j = 0; j < 3; ++j ){
		GridsTmp[j].resize( GRIDNUM );
		for( int i = 0; i < GRIDNUM; ++i ) GridsTmp[j][i].resize( GRIDNUM );
	}
}

ClothSim::~ClothSim()
{
}

void ClothSim::InitCloth()
{
	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			Grids[j][i].Mass = GRIDMASS;
			Grids[j][i].Pos.x = ( (float)i - (float)( GRIDNUM  - 1 ) / 2 ) * SPRINGLENGH;
			Grids[j][i].Pos.y = (float)j * SPRINGLENGH;
			Grids[j][i].Pos.z = 0;
		}
	}
}

void ClothSim::UpdateForce( GRIDS &Grids )
{
	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			if( ( i == 0 && j == GRIDNUM - 1 ) || ( i == GRIDNUM - 1 && j == GRIDNUM - 1 ) ) continue;

			GRID &g = Grids[j][i];

			// ̃Obhɓ͂̑aZo
			g.Frc = D3DXVECTOR3( 0, 0, 0 );
			
			// d
			g.Frc.y -= g.Mass * GRAVITYCONST;

			// ̂悤Ȃ
			float r = Counter / 70.0f; // K
			float r2 = Counter / 13.0f; // K

			g.Frc.x += cosf( r ) * sinf( r2 ) * WINDFORCE;
			g.Frc.z += sinf( r ) * sinf( r2 ) * WINDFORCE;

			// CR
			g.Frc -= g.Vel * DAMPINGCONST;

			// ȉAol֌W

			// ̃ol
			if( j <= GRIDNUM - 2 ){
				D3DXVECTOR3 d = g.Pos - Grids[j + 1][i].Pos;

				D3DXVECTOR3 n;
				D3DXVec3Normalize( &n, &d );

				g.Frc += n * ( ( SPRINGLENGH - D3DXVec3Length( &d ) ) * SPRINGCONST );
			}
			// ̃ol
			if( j >= 1 ){
				D3DXVECTOR3 d = g.Pos - Grids[j - 1][i].Pos;

				D3DXVECTOR3 n;
				D3DXVec3Normalize( &n, &d );

				g.Frc += n * ( ( SPRINGLENGH - D3DXVec3Length( &d ) ) * SPRINGCONST );
			}

			// ̃ol
			if( i >= 1 ){
				D3DXVECTOR3 d = g.Pos - Grids[j][i - 1].Pos;

				D3DXVECTOR3 n;
				D3DXVec3Normalize( &n, &d );

				g.Frc += n * ( ( SPRINGLENGH - D3DXVec3Length( &d ) ) * SPRINGCONST );
			}
			// Ẽol
			if( i <= GRIDNUM - 2 ){
				D3DXVECTOR3 d = g.Pos - Grids[j][i + 1].Pos;

				D3DXVECTOR3 n;
				D3DXVec3Normalize( &n, &d );

				g.Frc += n * ( ( SPRINGLENGH - D3DXVec3Length( &d ) ) * SPRINGCONST );
			}

			// ̃ol
			if( i >= 1 && j <= GRIDNUM - 2 ){
				D3DXVECTOR3 d = g.Pos - Grids[j + 1][i - 1].Pos;

				D3DXVECTOR3 n;
				D3DXVec3Normalize( &n, &d );

				g.Frc += n * ( ( DIAGONALSPRINGLENGTH - D3DXVec3Length( &d ) ) * DIAGONALSPRINGCONST );
			}

			// Ẽol
			if( i <= GRIDNUM - 2 && j <= GRIDNUM - 2 ){
				D3DXVECTOR3 d = g.Pos - Grids[j + 1][i + 1].Pos;

				D3DXVECTOR3 n;
				D3DXVec3Normalize( &n, &d );

				g.Frc += n * ( ( DIAGONALSPRINGLENGTH - D3DXVec3Length( &d ) ) * DIAGONALSPRINGCONST );
			}

			// ̃ol
			if( i >= 1 && j >= 1 ){
				D3DXVECTOR3 d = g.Pos - Grids[j - 1][i - 1].Pos;

				D3DXVECTOR3 n;
				D3DXVec3Normalize( &n, &d );

				g.Frc += n * ( ( DIAGONALSPRINGLENGTH - D3DXVec3Length( &d ) ) * DIAGONALSPRINGCONST );
			}

			// Ẽol
			if( i <= GRIDNUM - 2 && j >= 1 ){
				D3DXVECTOR3 d = g.Pos - Grids[j - 1][i + 1].Pos;

				D3DXVECTOR3 n;
				D3DXVec3Normalize( &n, &d );

				g.Frc += n * ( ( DIAGONALSPRINGLENGTH - D3DXVec3Length( &d ) ) * DIAGONALSPRINGCONST );
			}
		}
	}
}

void ClothSim::EularMethod()
{
	UpdateForce( Grids );

	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			if( ( i == 0 && j == GRIDNUM - 1 ) || ( i == GRIDNUM - 1 && j == GRIDNUM - 1 ) ) continue;

			Grids[j][i].Pos += Grids[j][i].Vel * DELTATIME;
			Grids[j][i].Vel += ( Grids[j][i].Frc / Grids[j][i].Mass ) * DELTATIME;
		}
	}
}

void ClothSim::FBEularMethod()
{
	UpdateForce( Grids );

	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			if( ( i == 0 && j == GRIDNUM - 1 ) || ( i == GRIDNUM - 1 && j == GRIDNUM - 1 ) ) continue;

			Grids[j][i].Vel += ( Grids[j][i].Frc / Grids[j][i].Mass ) * DELTATIME;
			Grids[j][i].Pos += Grids[j][i].Vel * DELTATIME;
		}
	}
}

void ClothSim::RK2Method() // _@
{
	UpdateForce( Grids );

	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			GridsTmp[0][j][i] = Grids[j][i];
		}
	}

	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			if( ( i == 0 && j == GRIDNUM - 1 ) || ( i == GRIDNUM - 1 && j == GRIDNUM - 1 ) ) continue;

			GridsTmp[0][j][i].Pos += GridsTmp[0][j][i].Vel * DELTATIME / 2;
			GridsTmp[0][j][i].Vel += ( GridsTmp[0][j][i].Frc / GridsTmp[0][j][i].Mass ) * DELTATIME / 2;
		}
	}

	UpdateForce( GridsTmp[0] );

	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			if( ( i == 0 && j == GRIDNUM - 1 ) || ( i == GRIDNUM - 1 && j == GRIDNUM - 1 ) ) continue;

			Grids[j][i].Pos += GridsTmp[0][j][i].Vel * DELTATIME;
			Grids[j][i].Vel += ( GridsTmp[0][j][i].Frc / GridsTmp[0][j][i].Mass ) * DELTATIME;
		}
	}
}

void ClothSim::RK4Method()
{
	UpdateForce( Grids ); // k1

	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			GridsTmp[0][j][i] = Grids[j][i];
		}
	}

	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			if( ( i == 0 && j == GRIDNUM - 1 ) || ( i == GRIDNUM - 1 && j == GRIDNUM - 1 ) ) continue;

			GridsTmp[0][j][i].Pos = Grids[j][i].Pos + GridsTmp[0][j][i].Vel * DELTATIME / 2;
			GridsTmp[0][j][i].Vel = Grids[j][i].Vel + ( GridsTmp[0][j][i].Frc / GridsTmp[0][j][i].Mass ) * DELTATIME / 2;
		}
	}

	UpdateForce( GridsTmp[0] ); // k2

	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			GridsTmp[1][j][i] = GridsTmp[0][j][i];
		}
	}

	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			if( ( i == 0 && j == GRIDNUM - 1 ) || ( i == GRIDNUM - 1 && j == GRIDNUM - 1 ) ) continue;

			GridsTmp[1][j][i].Pos = Grids[j][i].Pos + GridsTmp[1][j][i].Vel * DELTATIME / 2;
			GridsTmp[1][j][i].Vel = Grids[j][i].Vel + ( GridsTmp[1][j][i].Frc / GridsTmp[1][j][i].Mass ) * DELTATIME / 2;
		}
	}

	UpdateForce( GridsTmp[1] ); // k3

	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			GridsTmp[2][j][i] = GridsTmp[1][j][i];
		}
	}

	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			if( ( i == 0 && j == GRIDNUM - 1 ) || ( i == GRIDNUM - 1 && j == GRIDNUM - 1 ) ) continue;

			GridsTmp[2][j][i].Pos = Grids[j][i].Pos + GridsTmp[2][j][i].Vel * DELTATIME;
			GridsTmp[2][j][i].Vel = Grids[j][i].Vel + ( GridsTmp[2][j][i].Frc / GridsTmp[2][j][i].Mass ) * DELTATIME;
		}
	}

	UpdateForce( GridsTmp[2] ); // k4

	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			Grids[j][i].Pos += ( Grids[j][i].Vel + GridsTmp[0][j][i].Vel * 2 + GridsTmp[1][j][i].Vel * 2 + GridsTmp[2][j][i].Vel ) / 6 * DELTATIME;
			Grids[j][i].Vel += ( Grids[j][i].Frc + GridsTmp[0][j][i].Frc * 2 + GridsTmp[1][j][i].Frc * 2 + GridsTmp[2][j][i].Frc ) / Grids[j][i].Mass / 6 * DELTATIME;
		}
	}
}

void ClothSim::LeapFrogMethod()
{
	// [vtbO@̏lIC[@ŌvZĂ
	static IsInitialize = false;
	if( !IsInitialize ){
		for( int j = 0; j < GRIDNUM; ++j ){
			for( int i = 0; i < GRIDNUM; ++i ){
				GridsTmp[0][j][i] = Grids[j][i];
			}
		}

		UpdateForce( GridsTmp[0] );

		for( int j = 0; j < GRIDNUM; ++j ){
			for( int i = 0; i < GRIDNUM; ++i ){
				if( ( i == 0 && j == GRIDNUM - 1 ) || ( i == GRIDNUM - 1 && j == GRIDNUM - 1 ) ) continue;

				GridsTmp[0][j][i].Vel += ( GridsTmp[0][j][i].Frc / GridsTmp[0][j][i].Mass ) * DELTATIME / 2; // x0, v0.5, f0
			}
		}

		IsInitialize = true;
	}

	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			GridsTmp[0][j][i].Pos += GridsTmp[0][j][i].Vel * DELTATIME; // x1, v0.5, f0
		}
	}

	// CŘvZɕKvȑxdt/2Ô̂gĂ̂͂Aɕ@H
	UpdateForce( GridsTmp[0] ); // x1, v0.5, f1

	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			GridsTmp[0][j][i].Vel += GridsTmp[0][j][i].Frc / GridsTmp[0][j][i].Mass * DELTATIME; // x1, v1.5, f1
		}
	}

	for( int j = 0; j < GRIDNUM; ++j ){
		for( int i = 0; i < GRIDNUM; ++i ){
			Grids[j][i].Pos = GridsTmp[0][j][i].Pos; // x1
		}
	}
}

void ClothSim::UpdateCloth()
{
	for( int Loop = 0; Loop < RENDERINGTIME / DELTATIME; ++Loop ){
//		EularMethod();
		FBEularMethod();
//		RK2Method();
//		RK4Method();
//		LeapFrogMethod();
	}

	++Counter;
}