#ifndef __SIMCORE_H__
#define __SIMCORE_H__

#include <d3dx9.h>
#include <vector>

#define GRIDNUM 16
#define GRIDMASS 1.0f
#define SQRT2 1.41421356f
#define SPRINGLENGH 1.0f
#define DIAGONALSPRINGLENGTH ( SPRINGLENGH * SQRT2 )
#define SPRINGCONST 400.0f
#define DIAGONALSPRINGCONST 200.0f
#define DAMPINGCONST 0.2f
#define GRAVITYCONST 9.8f
#define DELTATIME 0.01f    // ڈ Eular: 0.0001f, FBEular: 0.01f, RK2Method: 0.001f, RK4Method: 0.01f LeapFrogMethod: 0.01f
#define RENDERINGTIME 0.1f
#define WINDFORCE 5.0f

struct GRID{
	float Mass;
	D3DXVECTOR3 Pos, Vel, Frc; // ʒuAxA

	GRID(){
		Pos = D3DXVECTOR3( 0, 0, 0 );
		Vel = D3DXVECTOR3( 0, 0, 0 );
		Frc = D3DXVECTOR3( 0, 0, 0 );
	}
};

typedef std::vector < std::vector < GRID > > GRIDS;

class ClothSim{
public:
	ClothSim();
	~ClothSim();
	void InitCloth();
	void UpdateCloth();
	const GRID& GetGrid( int x, int y ) const { return Grids[y][x]; }

private:
	void UpdateForce( GRIDS &Grids );
	void EularMethod();
	void FBEularMethod();
	void RK2Method();
	void RK4Method();
	void LeapFrogMethod();

private:
	GRIDS Grids;
	GRIDS GridsTmp[3];
	int Counter;
};

#endif //__SIMCORE_H__